#include <Wire.h>  //Нужна для переделки аналоговых ввводов A4 A5 в SDA SCL
#include <LiquidCrystal_PCF8574.h>
#include <EEPROM.h>
#include <dht11.h>
dht11 DHT;
#define PIN_DATA 3

void UpdateDisplay (int DopFunct = 0);
void UpdateRelleStat(int noStat = 0, int noUpdateDisplay = 0);  // Смена статуса релле в соответствии с температурой
int counter = -1;
int temperature = 0; 
int minTemp = 0;
int relle = 2; //вывод управления релле
int StatLampaRelle = 5; //Лампочка статуса релле
int StatRelle = 0; //статус релле ВЫКЛ
int LightSleepDef = 300; //Отключение подсветки дисплея через 5 мин. (300 сек.)
int LightSleep = LightSleepDef; //Отключение подсветки дисплея
int delayUpdDisplay = 0; //Анимация стрелочек при смене мин. температуры кнопками
int SmenaStatRelleDef = 120; //120 сек на задержку если скачет температура
int SmenaStatRelle = SmenaStatRelleDef;
int PressKeyDelayDef = 300; //300 мсек на задержку если нажата кнопка
int PressKeyDelay = PressKeyDelayDef;

LiquidCrystal_PCF8574 lcd(0x27); // Вариант для библиотеки PCF8574 

void setup() {
  Serial.begin(9600);
  pinMode(relle, OUTPUT); //релле
  pinMode(StatLampaRelle, OUTPUT); //Лампочка статуса релле
  EEPROM.get(2,minTemp);
  lcd.begin(16, 2);                     // Инициализация дисплея  
  lcd.setBacklight(true);               // Подключение подсветки
  lcd.clear();
  pinMode(14, OUTPUT); //кнопка '+' температура
  pinMode(15, OUTPUT); //кнопка '-' температура
  SmenaStatRelle = 5; //5 сек на раскачку датчика после запуска, чтоб не щёлкать релле по чём зря.
  LightSleep -=1; //Инициируем обратный отсчёт отключения дисплея
}


void loop() {  
if ((digitalRead(14)==HIGH)&&(PressKeyDelay == PressKeyDelayDef)){
  PressKeyDelay -= 1;  //Запускаем отсчёт задержки нажания клавиш
  if (LightSleep == LightSleepDef) {
    LightSleep -=1; //Если дисплей выключен - первое нажатие кнопки включает его
    UpdateDisplay(7);
    } else {
      LightSleep = LightSleepDef - 1; //"обнуляем" счётчик подсветки дисплея
      minTemp+=1;
      if (minTemp>40) minTemp=40;
      UpdateRelleStat(1,1); // Не блокировать релле на SmenaStatRelleDef сек., Не обновлять статус релле на дисплее
      UpdateDisplay(1);   //Обновить дисплей и отрисовать стрелочку
      EEPROM.put(2,minTemp);
    }
  }
if ((digitalRead(15)==HIGH)&&(PressKeyDelay == PressKeyDelayDef)){
  PressKeyDelay -= 1;  //Запускаем отсчёт задержки нажания клавиш  
  if (LightSleep == LightSleepDef) {
    LightSleep -=1; //Если дисплей выключен - первое нажатие кнопки включает его
    UpdateDisplay(7);
    } else {
      LightSleep = LightSleepDef - 1; //"обнуляем" счётчик подсветки дисплея
      minTemp-=1;
      if (minTemp<10) minTemp=10;
      UpdateRelleStat(1,1);  // Не блокировать релле на SmenaStatRelleDef сек., Не обновлять статус релле на дисплее
      UpdateDisplay(2);  //Обновить дисплей и отрисовать стрелочку
      EEPROM.put(2,minTemp);
    }
  }

 if (PressKeyDelay != PressKeyDelayDef) PressKeyDelay -= 1; //Обработка задержки нажатия кнопок
 if (PressKeyDelay <= 0) PressKeyDelay = PressKeyDelayDef;
 counter += 1;
 delay(1); //стопарим алгоритм на 1 мсек для более точного расчёта остальных задержек
   if ((counter % 1000) == 0) { // Цикл 1000 мс
    counter =0;
    if (LightSleep<LightSleepDef) LightSleep -=1;
    if (LightSleep <= 0) {
      LightSleep = LightSleepDef;
      UpdateDisplay(6);
      }
    if (SmenaStatRelle<SmenaStatRelleDef) SmenaStatRelle -= 1; //Задержка смены статуса релле на случай, если скачет температура
    if (SmenaStatRelle <= 0) {
      SmenaStatRelle = SmenaStatRelleDef;
      UpdateDisplay(5);   //Как вышла задержка - очистить отсчёт на дисплее
      }
    if (delayUpdDisplay>=0) { //Анимация стрелочек при установки температуры
      delayUpdDisplay -=1;
      if (delayUpdDisplay<0) UpdateDisplay(3); //Убрать стрелочки после задержки
      }
    DHT.read(PIN_DATA);
    if (temperature!=DHT.temperature) {
      temperature = DHT.temperature;
      UpdateDisplay(); //Если пришла новая температура - отобразим это на дисплее
    }
    UpdateRelleStat(); //Обновление статуса релле (если нужно)
    if (SmenaStatRelle != SmenaStatRelleDef) UpdateDisplay(4); //Отсчёт блокировки релле
   }
}

void UpdateDisplay (int DopFunct = 0){
switch (DopFunct) {
    case 1:
      lcd.setCursor(12,0);
      lcd.print(" ");
      lcd.setCursor(15,0);
      lcd.print(">");
      delayUpdDisplay = 1; //Анимация стрелочек при смене мин. температуры кнопками 1 сек.
      break;
    case 2:
      lcd.setCursor(15,0);
      lcd.print(" ");
      lcd.setCursor(12,0);
      lcd.print("<");
      delayUpdDisplay = 1; //Анимация стрелочек при смене мин. температуры кнопками 1 сек.
      break;
    case 3: // Очистка стрелок
      lcd.setCursor(12,0);
      lcd.print("    ");
      break;
    case 4: // Отсчёт последней смены статуса релле
      lcd.setCursor(11,1);
      lcd.print("     ");
      lcd.setCursor(11,1);
      lcd.print("<-");
      lcd.print(SmenaStatRelle);
      break;
    case 5: // Очистка отсчёта
      lcd.setCursor(11,1);
      lcd.print("     ");
      break;
    case 6: // Отключаем подсветку дисплея
      lcd.setBacklight(false);
      break;
    case 7: // Включаем подсветку дисплея
      lcd.setBacklight(true);
      break;
      
      
  }
  lcd.setCursor(0,0);              // Установка курсора в начало первой строки
  lcd.print("t:      ");       // Набор текста на первой строке
  lcd.setCursor(3,0);
  lcd.print(temperature);
  lcd.print(char(223));lcd.print("C");
  lcd.setCursor(8,0);
  lcd.print("min:");
  lcd.setCursor(13,0);
  lcd.print(minTemp);  
  if (StatRelle==1) {
    lcd.setCursor(0,1);
    lcd.print("Relle: on ");
    }else{
      lcd.setCursor(0,1);
      lcd.print("Relle: off");     
      }
}

void UpdateRelleStat(int noStat = 0, int noUpdateDisplay = 0) {
  if ((noStat != 1)&&(SmenaStatRelle != SmenaStatRelleDef)) return 0; //если релле ещё заблокировано, то выходим
  if ((temperature<=minTemp)&&(StatRelle == 0)) { //Смена статуса релле и обновление дисплея только если это нужно ...
    digitalWrite(relle, HIGH); 
    StatRelle = 1;
    analogWrite(StatLampaRelle,30); //Яркость лампочки поменьше, чтоб не святила ярко в глаз.
    if (noStat != 1) SmenaStatRelle -= 1;
    if (noUpdateDisplay != 1) UpdateDisplay();  
  }
  if ((temperature>minTemp)&&(StatRelle == 1)) { //... если релле и так в нужном положении, то ничего не меняем, дабы не напрягать дисплей
    digitalWrite(relle, LOW); 
    StatRelle = 0;
    analogWrite(StatLampaRelle,LOW);
    if (noStat != 1) SmenaStatRelle -= 1;
    if (noUpdateDisplay != 1) UpdateDisplay();  
  }
  return 0;  
}
